@@ -1,7 +1,6 @@
 """Module to do differentiable rendering of 2D and 3D scenes."""
-
 import copy
-
+import warnings
 
 import numpy as np
 
@@ -21,12 +20,12 @@ def renderScene(
 
     if check_valid:
         # doing checks here as it seems the debugger in not able to find the pyx file
-        # when installed from a wheel. this also make inderactive debugginh easier
+        # when installed from a wheel. this also make interactive debugging easier
         # for the library user
 
         assert not (image is None)
         assert not (z_buffer is None)
-        heigth = image.shape[0]
+        height = image.shape[0]
         width = image.shape[1]
         nb_colors = image.shape[2]
 
@@ -56,10 +55,18 @@ def renderScene(
         assert scene.edgeflags.shape[1] == 3
         assert scene.textured.shape[0] == nb_triangles
         assert scene.shaded.shape[0] == nb_triangles
-        assert scene.background.ndim == 3
-        assert scene.background.shape[0] == heigth
-        assert scene.background.shape[1] == width
-        assert scene.background.shape[2] == nb_colors
+
+        assert (scene.background_image is not None) != (
+            scene.background_color is not None
+        )
+
+        if scene.background_image is not None:
+            assert scene.background_image.ndim == 3
+            assert scene.background_image.shape[0] == height
+            assert scene.background_image.shape[1] == width
+            assert scene.background_image.shape[2] == nb_colors
+        else:
+            assert scene.background_color.shape[0] == nb_colors
 
         if scene.texture.size > 0:
             assert scene.texture.ndim == 3
@@ -67,13 +74,13 @@ def renderScene(
             assert scene.texture.shape[1] > 0
             assert scene.texture.shape[2] == nb_colors
 
-        assert z_buffer.shape[0] == heigth
+        assert z_buffer.shape[0] == height
         assert z_buffer.shape[1] == width
 
         if antialiase_error:
-            assert err_buffer.shape[0] == heigth
+            assert err_buffer.shape[0] == height
             assert err_buffer.shape[1] == width
-            assert obs.shape[0] == heigth
+            assert obs.shape[0] == height
             assert obs.shape[1] == width
             assert obs.shape[2] == nb_colors
 
@@ -103,13 +110,13 @@ def renderSceneB(
         assert not (image is None)
         assert not (z_buffer is None)
 
-        heigth = image.shape[0]
+        height = image.shape[0]
         width = image.shape[1]
         nb_colors = image.shape[2]
         nb_triangles = scene.faces.shape[0]
 
         assert nb_colors == scene.colors.shape[1]
-        assert z_buffer.shape[0] == heigth
+        assert z_buffer.shape[0] == height
         assert z_buffer.shape[1] == width
         assert nb_triangles == scene.faces_uv.shape[0]
 
@@ -137,10 +144,18 @@ def renderSceneB(
         assert scene.edgeflags.shape[1] == 3
         assert scene.textured.shape[0] == nb_triangles
         assert scene.shaded.shape[0] == nb_triangles
-        assert scene.background.ndim == 3
-        assert scene.background.shape[0] == heigth
-        assert scene.background.shape[1] == width
-        assert scene.background.shape[2] == nb_colors
+
+        assert (scene.background_image is not None) != (
+            scene.background_color is not None
+        )
+
+        if scene.background_image is not None:
+            assert scene.background_image.ndim == 3
+            assert scene.background_image.shape[0] == height
+            assert scene.background_image.shape[1] == width
+            assert scene.background_image.shape[2] == nb_colors
+        else:
+            assert scene.background_color.shape[0] == nb_colors
 
         assert scene.uv_b.ndim == 2
         assert scene.ij_b.ndim == 2
@@ -155,14 +170,6 @@ def renderSceneB(
         assert scene.shade_b.shape[0] == nb_vertices
         assert scene.colors_b.shape[0] == nb_vertices
         assert scene.colors_b.shape[1] == nb_colors
-        assert scene.edgeflags.shape[0] == nb_triangles
-        assert scene.edgeflags.shape[1] == 3
-        assert scene.textured.shape[0] == nb_triangles
-        assert scene.shaded.shape[0] == nb_triangles
-        assert scene.background.ndim == 3
-        assert scene.background.shape[0] == heigth
-        assert scene.background.shape[1] == width
-        assert scene.background.shape[2] == nb_colors
 
         if scene.texture.size > 0:
             assert scene.texture.ndim == 3
@@ -175,13 +182,13 @@ def renderSceneB(
             assert scene.texture_b.shape[2] == nb_colors
 
         if antialiase_error:
-            assert err_buffer.shape[0] == heigth
+            assert err_buffer.shape[0] == height
             assert err_buffer.shape[1] == width
-            assert obs.shape[0] == heigth
+            assert obs.shape[0] == height
             assert obs.shape[1] == width
         else:
             assert not (image_b is None)
-            assert image_b.shape[0] == heigth
+            assert image_b.shape[0] == height
             assert image_b.shape[1] == width
 
     differentiable_renderer_cython.renderSceneB(
@@ -257,7 +264,13 @@ class Camera:
             if store_backward is not None:
                 store_backward["project_points"] = (p_camera, depths, projected)
         else:
-            k1, k2, p1, p2, k3, = self.distortion
+            (
+                k1,
+                k2,
+                p1,
+                p2,
+                k3,
+            ) = self.distortion
             x = projected[:, 0]
             y = projected[:, 1]
             x2 = x ** 2
@@ -300,7 +313,13 @@ class Camera:
             p_camera, depths, projected, r2, radial_distortion = store_backward[
                 "project_points"
             ]
-            k1, k2, p1, p2, k3, = self.distortion
+            (
+                k1,
+                k2,
+                p1,
+                p2,
+                k3,
+            ) = self.distortion
             x = projected[:, 0]
             y = projected[:, 1]
             distorted_b = projected_image_coordinates_b.dot(
@@ -338,6 +357,16 @@ class Camera:
     def get_center(self):
         return -self.extrinsic[:3, :3].T.dot(self.extrinsic[:, 3])
 
+    def __repr__(self):
+        return (
+            f"<Camera>\n"
+            f"width:\n{str(self.width)}\n"
+            f"height:\n{str(self.height)}\n"
+            f"extrinsic:\n{str(self.extrinsic)}\n"
+            f"intrinsic:\n{str(self.intrinsic)}\n"
+            f"distortion:\n{str(self.distortion)}\n"
+        )
+
 
 class PerspectiveCamera(Camera):
     """Camera with perspective projection."""
@@ -415,11 +444,15 @@ class Scene2DBase:
         width,
         nb_colors,
         texture,
-        background,
+        background_image,
+        background_color,
         clockwise=False,
         backface_culling=True,
         strict_edge=True,
+        perspective_correct=False,
+        integer_pixel_centers=True,
     ):
+
         self.faces = faces
         self.faces_uv = faces_uv
         self.ij = ij
@@ -434,10 +467,13 @@ class Scene2DBase:
         self.width = width
         self.nb_colors = nb_colors
         self.texture = texture
-        self.background = background
+        self.background_image = background_image
+        self.background_color = background_color
         self.clockwise = clockwise
         self.backface_culling = backface_culling
         self.strict_edge = strict_edge
+        self.perspective_correct = perspective_correct
+        self.integer_pixel_centers = integer_pixel_centers
 
 
 class Scene2D(Scene2DBase):
@@ -461,11 +497,27 @@ class Scene2D(Scene2DBase):
         width,
         nb_colors,
         texture,
-        background,
+        background_image,
+        background_color,
         clockwise=False,
         backface_culling=False,
         strict_edge=True,
+        perspective_correct=False,
+        integer_pixel_centers=True,
     ):
+        """
+        Conventions:
+        If integer_pixel_centers is True (default) then pixel centers are at integer coordinates with
+            upper left at (0, 0)
+            upper right at (width - 1, 0)
+            lower left at (0, height - 1)
+            lower right at  (width - 1, height - 1)
+        if integer_pixel_centers is False, then pixel centers are at half integer coordinates with
+            upper left at (0.5, 0.5)
+            upper right at (width - 0.5, 0.5)
+            lower left at (0.5, height - 0.5)
+            lower right at  (width -0.5, height - 0.5)
+        """
         self.faces = faces
         self.faces_uv = faces_uv
         self.ij = ij
@@ -480,10 +532,13 @@ class Scene2D(Scene2DBase):
         self.width = width
         self.nb_colors = nb_colors
         self.texture = texture
-        self.background = background
+        self.background_image = background_image
+        self.background_color = background_color
         self.clockwise = clockwise
         self.backface_culling = backface_culling
         self.strict_edge = strict_edge
+        self.perspective_correct = perspective_correct
+        self.integer_pixel_centers = integer_pixel_centers
 
         # fields to store gradients
         self.uv_b = np.zeros(self.uv.shape)
@@ -517,6 +572,15 @@ class Scene2D(Scene2DBase):
         return image, z_buffer
 
     def render_error_backward(self, err_buffer_b, make_copies=True):
+        if self.perspective_correct:
+            raise BaseException(
+                "perspective_correct not supported yet for gradient back propagation"
+            )
+
+        if not self.backface_culling:
+            raise BaseException(
+                "use backface_culling=True if you use gradient backpropagation to get valid gradient through edge anti-aliasing."
+            )
         sigma, obs, image, z_buffer, err_buffer = self.store_backward
         antialiase_error = True
         if make_copies:
@@ -545,6 +609,14 @@ class Scene2D(Scene2DBase):
             )
 
     def render_backward(self, image_b, make_copies=True):
+        if self.perspective_correct:
+            raise BaseException(
+                "perspective_correct not supported yet for gradient back propagation"
+            )
+        if not self.backface_culling:
+            raise BaseException(
+                "use backface_culling=True if you use gradient backpropagation to get valid gradient through edge anti-aliasing."
+            )
         sigma, image, z_buffer = self.store_backward
         antialiase_error = False
         if (
@@ -584,6 +656,10 @@ class Scene2D(Scene2DBase):
         clear_gradients=True,
         make_copies=True,
     ):
+        if self.perspective_correct:
+            raise BaseException(
+                "perspective_correct not supported yet for gradient back propagation"
+            )
         if mask is None:
             mask = np.ones((obs.shape[0], obs.shape[1]))
         if antialiase_error:
@@ -615,11 +691,15 @@ class Scene3D:
     antialiasing edge overdraw.
     """
 
-    def __init__(self, sigma=1):
+    def __init__(self, sigma=1, perspective_correct=False, integer_pixel_centers=True):
         self.mesh = None
         self.light_directional = None
         self.light_ambient = None
         self.sigma = sigma
+        self.perspective_correct = perspective_correct
+        self.background_image = None
+        self.background_color = None
+        self.integer_pixel_centers = integer_pixel_centers
 
     def clear_gradients(self):
         # fields to store gradients
@@ -648,8 +728,31 @@ class Scene3D:
         self.mesh = mesh
 
     def set_background(self, background_image):
+        warnings.warn(
+            "This will be deprecated, please use set_background_image or set_background_color",
+            UserWarning,
+        )
+        self.set_background_image(background_image)
+
+    def set_background_image(self, background_image):
+        if self.background_color is not None:
+            raise BaseException(
+                "you cannot provide both background image and background color"
+            )
+        background_image = np.asanyarray(background_image)
         assert background_image.dtype == np.double
-        self.background = background_image
+        assert background_image.ndim == 3
+        self.background_image = background_image
+
+    def set_background_color(self, background_color):
+        if self.background_image is not None:
+            raise BaseException(
+                "you cannot provide both background image and background color"
+            )
+        background_color = np.asanyarray(background_color, dtype=np.float64)
+        assert background_color.dtype == np.double
+        assert background_color.ndim == 1
+        self.background_color = background_color
 
     def compute_vertices_luminosity(self):
         if self.light_directional is not None:
@@ -702,6 +805,7 @@ class Scene3D:
         z_buffer = np.empty((self.height, self.width))
         self.ij = np.array(ij)
         self.colors = np.array(colors)
+
         renderScene(self, self.sigma, image, z_buffer)
 
         if self.store_backward_current is not None:
@@ -710,6 +814,10 @@ class Scene3D:
         return image, z_buffer
 
     def _render_2d_backward(self, image_b):
+        if self.perspective_correct:
+            raise BaseException(
+                "perspective_correct not supported yet for gradient back propagation"
+            )
         ij, colors, image, z_buffer = self.store_backward_current["render_2d"]
         self.ij = np.array(ij)
         self.colors = np.array(colors)
@@ -722,6 +830,11 @@ class Scene3D:
         if self.light_directional is not None:
             self.mesh.compute_vertex_normals()
 
+        if (self.background_image is None) == (self.background_color is None):
+            raise BaseException(
+                "You need to provide either a background image or background color"
+            )
+
         points_2d, depths = camera.project_points(
             self.mesh.vertices, store_backward=self.store_backward_current
         )
@@ -761,6 +874,7 @@ class Scene3D:
         self.height = camera.height
         self.width = camera.width
         self.strict_edge = True
+
         self.clockwise = self.mesh.clockwise
         self.backface_culling = backface_culling
         image, z_buffer = self._render_2d(points_2d, colors)
@@ -776,6 +890,10 @@ class Scene3D:
             return image
 
     def render_backward(self, image_b):
+        if self.perspective_correct:
+            raise BaseException(
+                "perspective_correct not supported yet for gradient back propagation"
+            )
         camera, self.edgeflags = self.store_backward_current["render"]
         points_2d_b, colors_b = self._render_2d_backward(image_b)
         self._compute_vertices_colors_with_illumination_backward(colors_b)
@@ -822,6 +940,10 @@ class Scene3D:
         return image
 
     def render_depth_backward(self, depth_b):
+        if self.perspective_correct:
+            raise BaseException(
+                "perspective_correct not supported yet for gradient back propagation"
+            )
         camera, depth_scale = self.store_backward_current["render_depth"]
         ij_b, colors_b = self._render_2d_backward(depth_b)
         depths_b = np.squeeze(colors_b * depth_scale, axis=1)
@@ -931,9 +1053,11 @@ class Scene3D:
         )  # eventually used when using texture
         texture = np.zeros((0, 0))
 
-        background = np.zeros((height, width, nb_colors))
+        background_image = np.zeros((height, width, nb_colors))
         if "depth" in channels:
-            background[:, :, ranges["depth"][0] : ranges["depth"][1]] = depths.max()
+            background_image[
+                :, :, ranges["depth"][0] : ranges["depth"][1]
+            ] = depths.max()
 
         scene_2d = Scene2DBase(
             faces=soup_faces,
@@ -950,8 +1074,9 @@ class Scene3D:
             width=width,
             nb_colors=nb_colors,
             texture=texture,
-            background=background,
+            background_image=background_image,
             backface_culling=backface_culling,
+            background_color=None
         )
         buffers = np.empty((camera.height, camera.width, nb_colors))
         z_buffer = np.empty((camera.height, camera.width))
