@@ -8,7 +8,13 @@ import scipy.sparse.linalg
 import scipy.spatial.transform.rotation
 
 from . import Camera, ColoredTriMesh, LaplacianRigidEnergy, Scene3D, TriMesh
-from .tools import normalize, normalize_backward, qrot, qrot_backward
+from .tools import (
+    normalize,
+    normalize_backward,
+    qrot,
+    qrot_backward,
+    check_jacobian_finite_differences,
+)
 
 
 class MeshDepthFitter:
@@ -22,14 +28,13 @@ class MeshDepthFitter:
         translation_init,
         cregu=2000,
         inertia=0.96,
-        damping=0.05,
-        camera=None,
+        damping=0.05
     ):
         self.cregu = cregu
         self.inertia = inertia
         self.damping = damping
         self.step_factor_vertices = 0.0005
-        self.step_max_vertices = 0.5
+        self.step_max_vertices = 1
         self.step_factor_quaternion = 0.00006
         self.step_max_quaternion = 0.1
         self.step_factor_translation = 0.00005
@@ -68,18 +73,16 @@ class MeshDepthFitter:
 
     def set_max_depth(self, max_depth):
         self.scene.max_depth = max_depth
-        self.scene.set_background(
-            np.full((self.height, self.width, 1), max_depth, dtype=np.float)
-        )
+        self.scene.set_background_color(np.array([max_depth], dtype=np.float))
 
     def set_depth_scale(self, depth_scale):
         self.depthScale = depth_scale
 
-    def set_image(self, hand_image, focal=None, distortion=None):
-        self.width = hand_image.shape[1]
-        self.height = hand_image.shape[0]
-        assert hand_image.ndim == 2
-        self.hand_image = hand_image
+    def set_image(self, mesh_image, focal=None, distortion=None):
+        self.width = mesh_image.shape[1]
+        self.height = mesh_image.shape[0]
+        assert mesh_image.ndim == 2
+        self.mesh_image = mesh_image
         if focal is None:
             focal = 2 * self.width
         rot = np.array([[1, 0, 0], [0, -1, 0], [0, 0, -1]])
@@ -105,17 +108,16 @@ class MeshDepthFitter:
             qrot(q_normalized, self.vertices) + self.transform_translation
         )
         self.mesh.set_vertices(vertices_transformed)
-        self.depth_not_cliped = self.scene.render_depth(
-            self.camera,
-            depth_scale=self.depthScale,
+        self.depth_not_clipped = self.scene.render_depth(
+            self.camera, depth_scale=self.depthScale,
         )
-        depth = np.clip(self.depth_not_cliped, 0, self.scene.max_depth)
+        depth = np.clip(self.depth_not_clipped, 0, self.scene.max_depth)
         return depth
 
     def render_backward(self, depth_b):
         self.scene.clear_gradients()
-        depth_b[self.depth_not_cliped < 0] = 0
-        depth_b[self.depth_not_cliped > self.scene.max_depth] = 0
+        depth_b[self.depth_not_clipped < 0] = 0
+        depth_b[self.depth_not_clipped > self.scene.max_depth] = 0
         self.scene.render_depth_backward(depth_b)
         vertices_transformed_b = self.scene.mesh.vertices_b
         self.transform_translation_b = np.sum(vertices_transformed_b, axis=0)
@@ -133,9 +135,9 @@ class MeshDepthFitter:
         self.vertices = self.vertices - np.mean(self.vertices, axis=0)[None, :]
         depth = self.render()
 
-        diff_image = np.sum((depth - self.hand_image[:, :, None]) ** 2, axis=2)
+        diff_image = np.sum((depth - self.mesh_image[:, :, None]) ** 2, axis=2)
         energy_data = np.sum(diff_image)
-        depth_b = 2 * (depth - self.hand_image[:, :, None])
+        depth_b = 2 * (depth - self.mesh_image[:, :, None])
         self.render_backward(depth_b)
 
         self.vertices_b = self.vertices_b - np.mean(self.vertices_b, axis=0)[None, :]
@@ -145,7 +147,7 @@ class MeshDepthFitter:
         (
             energy_rigid,
             grad_rigidity,
-            approx_hessian_rigidity,
+            _,
         ) = self.rigid_energy.evaluate(self.vertices)
         energy = energy_data + energy_rigid
         print("Energy=%f : EData=%f E_rigid=%f" % (energy, energy_data, energy_rigid))
@@ -240,9 +242,7 @@ class MeshRGBFitterWithPose:
         self.reset()
 
     def set_background_color(self, background_color):
-        self.scene.set_background(
-            np.tile(background_color[None, None, :], (self.height, self.width, 1))
-        )
+        self.scene.set_background_color(background_color)
 
     def set_mesh_transform_init(self, euler, translation):
         self.transform_quaternion_init = scipy.spatial.transform.Rotation.from_euler(
@@ -258,19 +258,19 @@ class MeshRGBFitterWithPose:
         self.speed_translation = np.zeros(3)
         self.speed_quaternion = np.zeros(4)
 
-        self.hand_color = copy.copy(self.default_color)
+        self.mesh_color = copy.copy(self.default_color)
         self.light_directional = copy.copy(self.default_light["directional"])
         self.light_ambient = copy.copy(self.default_light["ambient"])
 
         self.speed_light_directional = np.zeros(self.light_directional.shape)
         self.speed_light_ambient = np.zeros(self.light_ambient.shape)
-        self.speed_hand_color = np.zeros(self.hand_color.shape)
+        self.speed_mesh_color = np.zeros(self.mesh_color.shape)
 
-    def set_image(self, hand_image, focal=None, distortion=None):
-        self.width = hand_image.shape[1]
-        self.height = hand_image.shape[0]
-        assert hand_image.ndim == 3
-        self.hand_image = hand_image
+    def set_image(self, mesh_image, focal=None, distortion=None):
+        self.width = mesh_image.shape[1]
+        self.height = mesh_image.shape[0]
+        assert mesh_image.ndim == 3
+        self.mesh_image = mesh_image
         if focal is None:
             focal = 2 * self.width
 
@@ -301,7 +301,7 @@ class MeshRGBFitterWithPose:
             light_directional=self.light_directional, light_ambient=self.light_ambient
         )
         self.mesh.set_vertices_colors(
-            np.tile(self.hand_color, (self.mesh.nb_vertices, 1))
+            np.tile(self.mesh_color, (self.mesh.nb_vertices, 1))
         )
         image = self.scene.render(self.camera)
         return image
@@ -309,7 +309,7 @@ class MeshRGBFitterWithPose:
     def render_backward(self, image_b):
         self.scene.clear_gradients()
         self.scene.render_backward(image_b)
-        self.hand_color_b = np.sum(self.mesh.vertices_colors_b, axis=0)
+        self.mesh_color_b = np.sum(self.mesh.vertices_colors_b, axis=0)
         self.light_directional_b = self.scene.light_directional_b
         self.light_ambient_b = self.scene.light_ambient_b
         vertices_transformed_b = self.scene.mesh.vertices_b
@@ -328,14 +328,14 @@ class MeshRGBFitterWithPose:
 
         image = self.render()
 
-        diff_image = np.sum((image - self.hand_image) ** 2, axis=2)
-        image_b = 2 * (image - self.hand_image)
+        diff_image = np.sum((image - self.mesh_image) ** 2, axis=2)
+        image_b = 2 * (image - self.mesh_image)
         energy_data = np.sum(diff_image)
 
         (
             energy_rigid,
             grad_rigidity,
-            approx_hessian_rigidity,
+            _,
         ) = self.rigid_energy.evaluate(self.vertices)
         energy = energy_data + energy_rigid
         print("Energy=%f : EData=%f E_rigid=%f" % (energy, energy_data, energy_rigid))
@@ -394,12 +394,12 @@ class MeshRGBFitterWithPose:
             self.speed_light_ambient * inertia + (1 - inertia) * step
         )
         self.light_ambient = self.light_ambient + self.speed_light_ambient
-        # update hand color
-        step = -self.hand_color_b * 0.00001
-        self.speed_hand_color = (1 - self.damping) * (
-            self.speed_hand_color * inertia + (1 - inertia) * step
+        # update mesh color
+        step = -self.mesh_color_b * 0.00001
+        self.speed_mesh_color = (1 - self.damping) * (
+            self.speed_mesh_color * inertia + (1 - inertia) * step
         )
-        self.hand_color = self.hand_color + self.speed_hand_color
+        self.mesh_color = self.mesh_color + self.speed_mesh_color
 
         self.iter += 1
         return energy, image, diff_image
@@ -417,20 +417,21 @@ class MeshRGBFitterWithPoseMultiFrame:
         default_color,
         default_light,
         cregu=2000,
-        inertia=0.96,
-        damping=0.05,
+        cdata=1,
+        inertia=0.97,
+        damping=0.15,
         update_lights=True,
         update_color=True,
     ):
         self.cregu = cregu
-
+        self.cdata = cdata
         self.inertia = inertia
         self.damping = damping
         self.step_factor_vertices = 0.0005
         self.step_max_vertices = 0.5
-        self.step_factor_quaternion = 0.00006
+        self.step_factor_quaternion = 0.00005
         self.step_max_quaternion = 0.05
-        self.step_factor_translation = 0.00005
+        self.step_factor_translation = 0.00004
         self.step_max_translation = 0.1
 
         self.default_color = default_color
@@ -452,12 +453,7 @@ class MeshRGBFitterWithPoseMultiFrame:
         self.reset()
 
     def set_background_color(self, background_color):
-        self.scene.set_background(
-            np.tile(
-                background_color[None, None, :].astype(np.float),
-                (self.height, self.width, 1),
-            )
-        )
+        self.scene.set_background_color(background_color)
 
     def set_mesh_transform_init(self, euler, translation):
         self.transform_quaternion_init = scipy.spatial.transform.Rotation.from_euler(
@@ -473,19 +469,19 @@ class MeshRGBFitterWithPoseMultiFrame:
         self.speed_translation = np.zeros(3)
         self.speed_quaternion = np.zeros(4)
 
-        self.hand_color = copy.copy(self.default_color)
+        self.mesh_color = copy.copy(self.default_color)
         self.light_directional = copy.copy(self.default_light["directional"])
         self.light_ambient = copy.copy(self.default_light["ambient"])
 
         self.speed_light_directional = np.zeros(self.light_directional.shape)
         self.speed_light_ambient = np.zeros(self.light_ambient.shape)
-        self.speed_hand_color = np.zeros(self.hand_color.shape)
+        self.speed_mesh_color = np.zeros(self.mesh_color.shape)
 
-    def set_images(self, hand_images, focal=None):
-        self.width = hand_images[0].shape[1]
-        self.height = hand_images[0].shape[0]
-        assert hand_images[0].ndim == 3
-        self.hand_images = hand_images
+    def set_images(self, mesh_images, focal=None):
+        self.width = mesh_images[0].shape[1]
+        self.height = mesh_images[0].shape[0]
+        assert mesh_images[0].ndim == 3
+        self.mesh_images = mesh_images
         if focal is None:
             focal = 2 * self.width
 
@@ -503,11 +499,11 @@ class MeshRGBFitterWithPoseMultiFrame:
         )
         self.iter = 0
 
-    def set_image(self, hand_image, focal=None):
-        self.width = hand_image.shape[1]
-        self.height = hand_image.shape[0]
-        assert hand_image.ndim == 3
-        self.hand_image = hand_image
+    def set_image(self, mesh_image, focal=None):
+        self.width = mesh_image.shape[1]
+        self.height = mesh_image.shape[0]
+        assert mesh_image.ndim == 3
+        self.mesh_image = mesh_image
         if focal is None:
             focal = 2 * self.width
 
@@ -538,7 +534,7 @@ class MeshRGBFitterWithPoseMultiFrame:
             light_directional=self.light_directional, light_ambient=self.light_ambient
         )
         self.mesh.set_vertices_colors(
-            np.tile(self.hand_color, (self.mesh.nb_vertices, 1))
+            np.tile(self.mesh_color, (self.mesh.nb_vertices, 1))
         )
         image = self.scene.render(self.camera)
         self.store_backward["render"] = (idframe, unormalized_quaternion, q_normalized)
@@ -550,14 +546,14 @@ class MeshRGBFitterWithPoseMultiFrame:
         self.vertices_b = np.zeros(self.vertices.shape)
         self.transform_quaternion_b = np.zeros(self.transform_quaternion.shape)
         self.transform_translation_b = np.zeros(self.transform_translation.shape)
-        self.hand_color_b = np.zeros(self.hand_color.shape)
+        self.mesh_color_b = np.zeros(self.mesh_color.shape)
         self.store_backward = {}
 
     def render_backward(self, image_b):
         idframe, unormalized_quaternion, q_normalized = self.store_backward["render"]
         self.scene.clear_gradients()
         self.scene.render_backward(image_b)
-        self.hand_color_b += np.sum(self.mesh.vertices_colors_b, axis=0)
+        self.mesh_color_b += np.sum(self.mesh.vertices_colors_b, axis=0)
         self.light_directional_b += self.scene.light_directional_b
         self.light_ambient_b += self.scene.light_ambient_b
         vertices_transformed_b = self.scene.mesh.vertices_b
@@ -571,35 +567,67 @@ class MeshRGBFitterWithPoseMultiFrame:
         )  # that will lead to a gradient that is in the tangeant space
         return
 
-    def step(self):
-        self.vertices = self.vertices - np.mean(self.vertices, axis=0)[None, :]
-
-        self.nb_facesrames = len(self.hand_images)
-
+    def energy_data(self, vertices, return_images=True):
+        self.vertices = vertices
         image = [None] * self.nb_facesrames
         diff_image = [None] * self.nb_facesrames
         image_b = [None] * self.nb_facesrames
         energy_datas = [None] * self.nb_facesrames
         self.clear_gradients()
-        coef_data = 1 / self.nb_facesrames
+        coef_data = self.cdata / self.nb_facesrames
         for idframe in range(self.nb_facesrames):
             image[idframe] = self.render(idframe=idframe)
             diff_image[idframe] = np.sum(
-                (image[idframe] - self.hand_images[idframe]) ** 2, axis=2
+                (image[idframe] - self.mesh_images[idframe]) ** 2, axis=2
             )
-            image_b = coef_data * 2 * (image[idframe] - self.hand_images[idframe])
+            image_b = coef_data * 2 * (image[idframe] - self.mesh_images[idframe])
             energy_datas[idframe] = coef_data * np.sum(diff_image[idframe])
             self.render_backward(image_b)
         energy_data = np.sum(energy_datas)
+        if return_images:
+            return energy_data, image, diff_image
+        else:
+            return energy_data
+
+    def step(self, check_gradient=False):
+
+        self.vertices = self.vertices - np.mean(self.vertices, axis=0)[None, :]
+
+        self.nb_facesrames = len(self.mesh_images)
+
+        energy_data, image, diff_image = self.energy_data(self.vertices)
         (
             energy_rigid,
             grad_rigidity,
-            approx_hessian_rigidity,
+            _,
         ) = self.rigid_energy.evaluate(self.vertices)
-        energy = energy_data + energy_rigid
-        print("Energy=%f : EData=%f E_rigid=%f" % (energy, energy_data, energy_rigid))
 
-        self.vertices_b = self.vertices_b - np.mean(self.vertices_b, axis=0)[None, :]
+        if check_gradient:
+
+            def func(x):
+                return self.rigid_energy.evaluate(
+                    x, return_grad=False, return_hessian=False
+                )
+
+            check_jacobian_finite_differences(
+                grad_rigidity.flatten(), func, self.vertices
+            )
+
+            def func(x):
+                return self.energy_data(x, return_images=False)
+
+            grad_data = self.vertices_b.copy()
+            check_jacobian_finite_differences(grad_data.flatten(), func, self.vertices)
+
+        energy = energy_data + energy_rigid
+        print(
+            f"iter {self.iter} Energy={energy} : EData={energy_data} E_rigid={energy_rigid}"
+        )
+
+        if self.iter < 500:
+            self.vertices_b = (
+                self.vertices_b - np.mean(self.vertices_b, axis=0)[None, :]
+            )
         # update v
         grad = self.vertices_b + grad_rigidity
 
@@ -612,11 +640,13 @@ class MeshRGBFitterWithPoseMultiFrame:
         step_vertices = mult_and_clamp(
             -grad, self.step_factor_vertices, self.step_max_vertices
         )
+
         self.speed_vertices = (1 - self.damping) * (
             self.speed_vertices * inertia + (1 - inertia) * step_vertices
         )
         self.vertices = self.vertices + self.speed_vertices
         # update rotation
+
         step_quaternion = mult_and_clamp(
             -self.transform_quaternion_b,
             self.step_factor_quaternion,
@@ -651,12 +681,12 @@ class MeshRGBFitterWithPoseMultiFrame:
             self.speed_light_ambient * inertia + (1 - inertia) * step
         )
         self.light_ambient = self.light_ambient + self.speed_light_ambient
-        # update hand color
-        step = -self.hand_color_b * 0.00001
-        self.speed_hand_color = (1 - self.damping) * (
-            self.speed_hand_color * inertia + (1 - inertia) * step
+        # update mesh color
+        step = -self.mesh_color_b * 0.00001
+        self.speed_mesh_color = (1 - self.damping) * (
+            self.speed_mesh_color * inertia + (1 - inertia) * step
         )
-        self.hand_color = self.hand_color + self.speed_hand_color
+        self.mesh_color = self.mesh_color + self.speed_mesh_color
 
         self.iter += 1
         return energy, image, diff_image
