@@ -66,18 +66,16 @@ class MeshDepthFitterEnergy(torch.nn.Module):
 
     def set_max_depth(self, max_depth):
         self.scene.max_depth = max_depth
-        self.scene.set_background(
-            np.full((self.height, self.width, 1), max_depth, dtype=np.float)
-        )
+        self.scene.set_background_color(np.array([max_depth], dtype=np.float))
 
     def set_depth_scale(self, depth_scale):
         self.depthScale = depth_scale
 
-    def set_image(self, hand_image, focal=None, distortion=None):
-        self.width = hand_image.shape[1]
-        self.height = hand_image.shape[0]
-        assert hand_image.ndim == 2
-        self.hand_image = hand_image
+    def set_image(self, mesh_image, focal=None, distortion=None):
+        self.width = mesh_image.shape[1]
+        self.height = mesh_image.shape[0]
+        assert mesh_image.ndim == 2
+        self.mesh_image = mesh_image
         if focal is None:
             focal = 2 * self.width
 
@@ -107,7 +105,7 @@ class MeshDepthFitterEnergy(torch.nn.Module):
         )
         depth = torch.clamp(depth, 0, self.scene.max_depth)
         diff_image = torch.sum(
-            (depth - torch.tensor(self.hand_image[:, :, None])) ** 2, dim=2
+            (depth - torch.tensor(self.mesh_image[:, :, None])) ** 2, dim=2
         )
         self.depth = depth
         self.diff_image = diff_image
@@ -220,18 +218,16 @@ class MeshDepthFitter:
 
     def set_max_depth(self, max_depth):
         self.scene.max_depth = max_depth
-        self.scene.set_background(
-            np.full((self.height, self.width, 1), max_depth, dtype=np.float)
-        )
+        self.scene.set_background_color([max_depth])
 
     def set_depth_scale(self, depth_scale):
         self.depthScale = depth_scale
 
-    def set_image(self, hand_image, focal=None, distortion=None):
-        self.width = hand_image.shape[1]
-        self.height = hand_image.shape[0]
-        assert hand_image.ndim == 2
-        self.hand_image = hand_image
+    def set_image(self, mesh_image, focal=None, distortion=None):
+        self.width = mesh_image.shape[1]
+        self.height = mesh_image.shape[0]
+        assert mesh_image.ndim == 2
+        self.mesh_image = mesh_image
         if focal is None:
             focal = 2 * self.width
 
@@ -266,7 +262,7 @@ class MeshDepthFitter:
 
         q_normalized = (
             quaternion_with_grad / quaternion_with_grad.norm()
-        )  # that will lead to a gradient that is in the tangeant space
+        )  # that will lead to a gradient that is in the tangent space
         vertices_with_grad_transformed = (
             qrot(q_normalized, vertices_with_grad_centered) + translation_with_grad
         )
@@ -274,13 +270,11 @@ class MeshDepthFitter:
         self.mesh.set_vertices(vertices_with_grad_transformed)
 
         depth_scale = 1 * self.depthScale
-        depth = self.scene.render_depth(
-            self.camera, depth_scale=depth_scale
-        )
+        depth = self.scene.render_depth(self.camera, depth_scale=depth_scale)
         depth = torch.clamp(depth, 0, self.scene.max_depth)
 
         diff_image = torch.sum(
-            (depth - torch.tensor(self.hand_image[:, :, None])) ** 2, dim=2
+            (depth - torch.tensor(self.mesh_image[:, :, None])) ** 2, dim=2
         )
         loss = torch.sum(diff_image)
 
@@ -292,7 +286,7 @@ class MeshDepthFitter:
         (
             energy_rigid,
             grad_rigidity,
-            approx_hessian_rigidity,
+            _,
         ) = self.rigid_energy.evaluate(self.vertices.numpy())
         energy = energy_data + energy_rigid
         print("Energy=%f : EData=%f E_rigid=%f" % (energy, energy_data, energy_rigid))
@@ -390,9 +384,7 @@ class MeshRGBFitterWithPose:
         self.reset()
 
     def set_background_color(self, background_color):
-        self.scene.set_background(
-            np.tile(background_color[None, None, :], (self.height, self.width, 1))
-        )
+        self.scene.set_background_color(background_color)
 
     def set_mesh_transform_init(self, euler, translation):
         self.transform_quaternion_init = scipy.spatial.transform.Rotation.from_euler(
@@ -408,19 +400,19 @@ class MeshRGBFitterWithPose:
         self.speed_translation = np.zeros(3)
         self.speed_quaternion = np.zeros(4)
 
-        self.hand_color = copy.copy(self.default_color)
+        self.mesh_color = copy.copy(self.default_color)
         self.light_directional = copy.copy(self.default_light["directional"])
         self.light_ambient = copy.copy(self.default_light["ambient"])
 
         self.speed_light_directional = np.zeros(self.light_directional.shape)
         self.speed_light_ambient = np.zeros(self.light_ambient.shape)
-        self.speed_hand_color = np.zeros(self.hand_color.shape)
+        self.speed_mesh_color = np.zeros(self.mesh_color.shape)
 
-    def set_image(self, hand_image, focal=None, distortion=None):
-        self.width = hand_image.shape[1]
-        self.height = hand_image.shape[0]
-        assert hand_image.ndim == 3
-        self.hand_image = hand_image
+    def set_image(self, mesh_image, focal=None, distortion=None):
+        self.width = mesh_image.shape[1]
+        self.height = mesh_image.shape[0]
+        assert mesh_image.ndim == 3
+        self.mesh_image = mesh_image
         if focal is None:
             focal = 2 * self.width
 
@@ -458,13 +450,13 @@ class MeshRGBFitterWithPose:
         light_ambient_with_grad = torch.tensor(
             self.light_ambient, dtype=torch.float64, requires_grad=True
         )
-        hand_color_with_grad = torch.tensor(
-            self.hand_color, dtype=torch.float64, requires_grad=True
+        mesh_color_with_grad = torch.tensor(
+            self.mesh_color, dtype=torch.float64, requires_grad=True
         )
 
         q_normalized = (
             quaternion_with_grad / quaternion_with_grad.norm()
-        )  # that will lead to a gradient that is in the tangeant space
+        )  # that will lead to a gradient that is in the tangent space
         vertices_with_grad_transformed = (
             qrot(q_normalized, vertices_with_grad_centered) + translation_with_grad
         )
@@ -475,12 +467,12 @@ class MeshRGBFitterWithPose:
             light_ambient=light_ambient_with_grad,
         )
         self.mesh.set_vertices_colors(
-            hand_color_with_grad.repeat([self.mesh.nb_vertices, 1])
+            mesh_color_with_grad.repeat([self.mesh.nb_vertices, 1])
         )
 
         image = self.scene.render(self.camera)
 
-        diff_image = torch.sum((image - torch.tensor(self.hand_image)) ** 2, dim=2)
+        diff_image = torch.sum((image - torch.tensor(self.mesh_image)) ** 2, dim=2)
         loss = torch.sum(diff_image)
 
         loss.backward()
@@ -548,12 +540,12 @@ class MeshRGBFitterWithPose:
             self.speed_light_ambient * inertia + (1 - inertia) * step
         )
         self.light_ambient = self.light_ambient + self.speed_light_ambient
-        # update hand color
-        step = -hand_color_with_grad.grad.numpy() * 0.00001
-        self.speed_hand_color = (1 - self.damping) * (
-            self.speed_hand_color * inertia + (1 - inertia) * step
+        # update mesh color
+        step = -mesh_color_with_grad.grad.numpy() * 0.00001
+        self.speed_mesh_color = (1 - self.damping) * (
+            self.speed_mesh_color * inertia + (1 - inertia) * step
         )
-        self.hand_color = self.hand_color + self.speed_hand_color
+        self.mesh_color = self.mesh_color + self.speed_mesh_color
 
         self.iter += 1
         return energy, image.detach().numpy(), diff_image.detach().numpy()
