@@ -1,4 +1,4 @@
-"""Tensorflow implementation of an as-rigi-as-possible energy based on the difference of laplacian with a reference shape."""
+"""Tensorflow implementation of an as-rigid-as-possible energy based on the difference of laplacian with a reference shape."""
 
 import tensorflow as tf
 
@@ -7,7 +7,7 @@ from ..laplacian_rigid_energy import LaplacianRigidEnergy
 
 
 class LaplacianRigidEnergyTensorflow(LaplacianRigidEnergy):
-    """Tensorflow class that implements an as-rigi-as-possible energy based on the difference of laplacian with a reference shape."""
+    """Tensorflow class that implements an as-rigid-as-possible energy based on the difference of laplacian with a reference shape."""
 
     def __init__(self, mesh, vertices, cregu):
         super().__init__(mesh, vertices, cregu)
